var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

function _objectWithoutProperties(obj, keys) { var target = {}; for (var i in obj) { if (keys.indexOf(i) >= 0) continue; if (!Object.prototype.hasOwnProperty.call(obj, i)) continue; target[i] = obj[i]; } return target; }

import h337 from 'heatmap.js';
import debounce from 'lodash.debounce';
import { addListener } from 'resize-detector';
import { ZOOM_TYPE } from './constants';

export default {
  props: {
    heatmapOptions: {
      type: Object,
      default: function _default() {
        return {};
      }
    },
    heatmapMin: {
      type: Number,
      default: 0
    },
    heatmapMax: {
      type: Number,
      default: 100
    },
    heatmapData: {
      type: Array,
      default: function _default() {
        return [];
      }
    },
    heatmapRadiusZoom: {
      type: Boolean,
      default: false
    },
    heatmapClickDrawable: {
      type: Boolean,
      default: false
    },
    heatmapMoveDrawable: {
      type: Boolean,
      default: false
    },
    heatmapDrawValue: {
      type: Number,
      default: 1
    }
  },
  computed: {
    mHeatmapOptions: function mHeatmapOptions() {
      return _extends({
        radius: 40 }, this.heatmapOptions);
    }
  },
  methods: {
    // 在组件挂载的时候会被调用
    initHeatMap: function initHeatMap() {
      this.$heatmapEl = this.$el.querySelector('.canvas-container');
      this.heatmap = h337.create({
        container: this.$heatmapEl
      });
      addListener(this.$heatmapEl, this._handleHeatmapResize);
      this.renderHeatmap();
      this.canvas.on('mouse:up', this.handleCanvasMouseUpForHeatmap);
      this.canvas.on('object:scaling', this.handleCanvasScalingForHeatmap);
      this.canvas.on('mouse:move', this.handleCanvasMouseMoveForHeatmap);
    },
    _handleHeatmapResize: function _handleHeatmapResize() {
      if (!this.$heatmapEl) return;
      this.heatmap.configure(_extends({}, this.mHeatmapOptions, {
        width: this.$heatmapEl.clientWidth,
        height: this.$heatmapEl.clientHeight
      }));
    },
    handleCanvasMouseUpForHeatmap: function handleCanvasMouseUpForHeatmap(opt) {
      if (opt.target === this.svgMap) {
        if (this.heatmapClickDrawable && this.heatmap && opt.pointer) {
          // 根据坐标来判断，当前是处于拖动状态，还是点击状态
          if (this.svgMap.left === this.mouseDownSvgMapPointer.left && this.svgMap.top === this.mouseDownSvgMapPointer.top) {
            var _opt$pointer = opt.pointer,
                x = _opt$pointer.x,
                y = _opt$pointer.y;

            this.requestCallAdd({ x: x, y: y });
          }
        }
      }
    },
    handleCanvasScalingForHeatmap: function handleCanvasScalingForHeatmap(opt) {
      if (opt.target === this.svgMap) {
        this.renderHeatmap();
      }
    },
    handleCanvasMouseMoveForHeatmap: function handleCanvasMouseMoveForHeatmap(opt) {
      if (opt.target === this.svgMap) {
        if (this.heatmapMoveDrawable && this.heatmap && opt.pointer) {
          var _opt$pointer2 = opt.pointer,
              x = _opt$pointer2.x,
              y = _opt$pointer2.y;

          this.requestCallAdd({ x: x, y: y });
        }
        this.renderHeatmap();
      }
    },

    requestCallAdd: debounce(function () {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          x = _ref.x,
          y = _ref.y;

      this.callAdd({ x: x, y: y });
    }, 15),
    callAdd: function callAdd() {
      var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          x = _ref2.x,
          y = _ref2.y;

      var _point2svgRelativeRat = this.point2svgRelativeRateInfo({ x: x, y: y }, true),
          coordX = _point2svgRelativeRat.coordX,
          coordY = _point2svgRelativeRat.coordY;

      this.callFun('heatmapAddCb', { coordX: coordX, coordY: coordY, value: this.heatmapDrawValue });
    },
    renderHeatmap: function renderHeatmap() {
      var _this = this;

      var data = this.heatmapData.map(function (item) {
        var _item$coordX = item.coordX,
            coordX = _item$coordX === undefined ? 0 : _item$coordX,
            _item$coordY = item.coordY,
            coordY = _item$coordY === undefined ? 0 : _item$coordY,
            radius = item.radius,
            rest = _objectWithoutProperties(item, ['coordX', 'coordY', 'radius']);

        var _svgRateInfo2Point = _this.svgRateInfo2Point({ coordX: coordX, coordY: coordY }, true),
            x = _svgRateInfo2Point.x,
            y = _svgRateInfo2Point.y;

        var _radius = radius || _this.mHeatmapOptions.radius;

        // 热力图在缩放的时候是否能够跟随缩放，不会因缩放而导致热力集中或分散
        if (_this.heatmapRadiusZoom && _this.svgMap && _this.canvas) {
          if (_this.zoomType === ZOOM_TYPE.MAP) {
            _radius = _radius * _this.svgMap.scaleX;
          } else {
            _radius = _radius * _this.canvas.getZoom();
          }
        }

        return _extends({
          x: x, y: y,
          radius: _radius
        }, rest);
      });
      this.heatmap && this.heatmap.setData({
        max: this.heatmapMax,
        min: this.heatmapMin,
        data: data
      });
    },
    heatmapAddData: function heatmapAddData(val) {
      return this.heatmap && this.heatmap.addData(val);
    },
    heatmapGetValueAt: function heatmapGetValueAt(val) {
      return this.heatmap && this.heatmap.getValueAt(val);
    },
    heatmapGetData: function heatmapGetData() {
      return this.heatmap && this.heatmap.getData();
    },
    heatmapGetDataURL: function heatmapGetDataURL() {
      return this.heatmap && this.heatmap.getDataURL();
    },
    heatmapRepaint: function heatmapRepaint() {
      return this.heatmap && this.heatmap.repaint();
    }
  },
  watch: {
    heatmapData: function heatmapData() {
      this.renderHeatmap();
    },
    heatmapRadiusZoom: function heatmapRadiusZoom() {
      this.renderHeatmap();
    },
    mHeatmapOptions: function mHeatmapOptions(val) {
      this.heatmap && this.heatmap.configure(val);
    },
    heatmapMin: function heatmapMin(val) {
      this.heatmap && this.heatmap.setDataMin(val);
    },
    heatmapMax: function heatmapMax(val) {
      this.heatmap && this.heatmap.setDataMax(val);
    }
  }
};